@@ -30,7 +30,6 @@ class BaseQSA(Optimizer):
     >>>     "lb": [-10, -15, -4, -2, -8],
     >>>     "ub": [10, 15, 12, 8, 20],
     >>>     "minmax": "min",
-    >>>     "verbose": True,
     >>> }
     >>>
     >>> epoch = 1000
@@ -48,17 +47,17 @@ class BaseQSA(Optimizer):
             pop_size (int): number of population size, default = 100
         """
         super().__init__(problem, kwargs)
-        self.nfe_per_epoch = 3 * pop_size
+        self.epoch = self.validator.check_int("epoch", epoch, [1, 100000])
+        self.pop_size = self.validator.check_int("pop_size", pop_size, [10, 10000])
+        self.nfe_per_epoch = 3 * self.pop_size
         self.sort_flag = True
 
-        self.epoch = epoch
-        self.pop_size = pop_size
-
-    def _calculate_queue_length__(self, t1, t2, t3):
-        """ Calculate length of each queue based on  t1, t2,t3
-                t1 = t1 * 1.0e+100
-                t2 = t2 * 1.0e+100
-                t3 = t3 * 1.0e+100
+    def calculate_queue_length__(self, t1, t2, t3):
+        """
+        Calculate length of each queue based on  t1, t2,t3
+            + t1 = t1 * 1.0e+100
+            + t2 = t2 * 1.0e+100
+            + t3 = t3 * 1.0e+100
         """
         if t1 > 1.0e-6:
             n1 = (1 / t1) / ((1 / t1) + (1 / t2) + (1 / t3))
@@ -71,10 +70,10 @@ class BaseQSA(Optimizer):
         q3 = self.pop_size - q1 - q2
         return q1, q2, q3
 
-    def _update_business_1(self, pop=None, current_epoch=None):
+    def update_business_1__(self, pop=None, current_epoch=None):
         A1, A2, A3 = pop[0][self.ID_POS], pop[1][self.ID_POS], pop[2][self.ID_POS]
         t1, t2, t3 = pop[0][self.ID_TAR][self.ID_FIT], pop[1][self.ID_TAR][self.ID_FIT], pop[2][self.ID_TAR][self.ID_FIT]
-        q1, q2, q3 = self._calculate_queue_length__(t1, t2, t3)
+        q1, q2, q3 = self.calculate_queue_length__(t1, t2, t3)
         case = None
         for i in range(self.pop_size):
             if i < q1:
@@ -96,27 +95,27 @@ class BaseQSA(Optimizer):
             F2 = beta * alpha * (E * np.abs(A - pop[i][self.ID_POS]))
             if case == 1:
                 pos_new = A + F1
-                pos_new = self.amend_position_faster(pos_new)
-                fit_new = self.get_fitness_position(pos_new)
-                if self.compare_agent([pos_new, fit_new], pop[i]):
-                    pop[i] = [pos_new, fit_new]
+                pos_new = self.amend_position(pos_new, self.problem.lb, self.problem.ub)
+                target = self.get_target_wrapper(pos_new)
+                if self.compare_agent([pos_new, target], pop[i]):
+                    pop[i] = [pos_new, target]
                 else:
                     case = 2
             else:
                 pos_new = pop[i][self.ID_POS] + F2
-                pos_new = self.amend_position_faster(pos_new)
-                fit_new = self.get_fitness_position(pos_new)
-                if self.compare_agent([pos_new, fit_new], pop[i]):
-                    pop[i] = [pos_new, fit_new]
+                pos_new = self.amend_position(pos_new, self.problem.lb, self.problem.ub)
+                target = self.get_target_wrapper(pos_new)
+                if self.compare_agent([pos_new, target], pop[i]):
+                    pop[i] = [pos_new, target]
                 else:
                     case = 1
         pop, _ = self.get_global_best_solution(pop)
         return pop
 
-    def _update_business_2(self, pop=None):
+    def update_business_2__(self, pop=None):
         A1, A2, A3 = pop[0][self.ID_POS], pop[1][self.ID_POS], pop[2][self.ID_POS]
         t1, t2, t3 = pop[0][self.ID_TAR][self.ID_FIT], pop[1][self.ID_TAR][self.ID_FIT], pop[2][self.ID_TAR][self.ID_FIT]
-        q1, q2, q3 = self._calculate_queue_length__(t1, t2, t3)
+        q1, q2, q3 = self.calculate_queue_length__(t1, t2, t3)
         pr = [i / self.pop_size for i in range(1, self.pop_size + 1)]
         if t1 > 1.0e-005:
             cv = t1 / (t2 + t3)
@@ -136,16 +135,19 @@ class BaseQSA(Optimizer):
                     X_new = pop[i][self.ID_POS] + np.random.exponential(0.5) * (pop[i1][self.ID_POS] - pop[i2][self.ID_POS])
                 else:
                     X_new = pop[i][self.ID_POS] + np.random.exponential(0.5) * (A - pop[i1][self.ID_POS])
-                pos_new = self.amend_position_faster(X_new)
             else:
-                pos_new = np.random.uniform(self.problem.lb, self.problem.ub)
+                X_new = self.generate_position(self.problem.lb, self.problem.ub)
+            pos_new = self.amend_position(X_new, self.problem.lb, self.problem.ub)
             pop_new.append([pos_new, None])
-        pop_new = self.update_fitness_population(pop_new)
-        pop = self.greedy_selection_population(pop, pop_new)
-        pop, _ = self.get_global_best_solution(pop)
-        return pop
+            if self.mode not in self.AVAILABLE_MODES:
+                target = self.get_target_wrapper(pos_new)
+                pop_new[-1] = self.get_better_solution([pos_new, target], pop[i])
+        if self.mode in self.AVAILABLE_MODES:
+            pop_new = self.update_target_wrapper_population(pop_new)
+            pop_new = self.greedy_selection_population(pop, pop_new)
+        return self.get_sorted_strim_population(pop_new, self.pop_size)
 
-    def _update_business_3(self, pop, g_best):
+    def update_business_3__(self, pop, g_best):
         pr = np.array([i / self.pop_size for i in range(1, self.pop_size + 1)])
         pop_new = []
         for i in range(self.pop_size):
@@ -153,10 +155,14 @@ class BaseQSA(Optimizer):
             id1 = np.random.choice(self.pop_size)
             temp = g_best[self.ID_POS] + np.random.exponential(0.5, self.problem.n_dims) * (pop[id1][self.ID_POS] - pop[i][self.ID_POS])
             X_new = np.where(np.random.random(self.problem.n_dims) > pr[i], temp, X_new)
-            pos_new = self.amend_position_faster(X_new)
+            pos_new = self.amend_position(X_new, self.problem.lb, self.problem.ub)
             pop_new.append([pos_new, None])
-        pop_new = self.update_fitness_population(pop_new)
-        pop_new = self.greedy_selection_population(pop, pop_new)
+            if self.mode not in self.AVAILABLE_MODES:
+                target = self.get_target_wrapper(pos_new)
+                pop_new[-1] = self.get_better_solution([pos_new, target], pop[i])
+        if self.mode in self.AVAILABLE_MODES:
+            pop_new = self.update_target_wrapper_population(pop_new)
+            pop_new = self.greedy_selection_population(pop, pop_new)
         return pop_new
 
     def evolve(self, epoch):
@@ -166,9 +172,9 @@ class BaseQSA(Optimizer):
         Args:
             epoch (int): The current iteration
         """
-        pop = self._update_business_1(self.pop, epoch + 1)
-        pop = self._update_business_2(pop)
-        self.pop = self._update_business_3(pop, self.g_best)
+        pop = self.update_business_1__(self.pop, epoch + 1)
+        pop = self.update_business_2__(pop)
+        self.pop = self.update_business_3__(pop, self.g_best)
 
 
 class OppoQSA(BaseQSA):
@@ -192,7 +198,6 @@ class OppoQSA(BaseQSA):
     >>>     "lb": [-10, -15, -4, -2, -8],
     >>>     "ub": [10, 15, 12, 8, 20],
     >>>     "minmax": "min",
-    >>>     "verbose": True,
     >>> }
     >>>
     >>> epoch = 1000
@@ -210,18 +215,23 @@ class OppoQSA(BaseQSA):
             pop_size (int): number of population size, default = 100
         """
         super().__init__(problem, epoch, pop_size, **kwargs)
-        self.nfe_per_epoch = 4 * pop_size
+        self.nfe_per_epoch = 4 * self.pop_size
         self.sort_flag = True
 
-    def _opposition_based(self, pop=None, g_best=None):
+    def opposition_based__(self, pop=None, g_best=None):
         pop, _ = self.get_global_best_solution(pop)
         pop_new = []
         for i in range(0, self.pop_size):
             X_new = self.create_opposition_position(pop[i], g_best)
-            pos_new = self.amend_position_faster(X_new)
+            pos_new = self.amend_position(X_new, self.problem.lb, self.problem.ub)
             pop_new.append([pos_new, None])
-        pop_new = self.update_fitness_population(pop_new)
-        return self.greedy_selection_population(pop, pop_new)
+            if self.mode not in self.AVAILABLE_MODES:
+                target = self.get_target_wrapper(pos_new)
+                pop_new[-1] = self.get_better_solution([pos_new, target], pop[i])
+        if self.mode in self.AVAILABLE_MODES:
+            pop_new = self.update_target_wrapper_population(pop_new)
+            pop_new = self.greedy_selection_population(pop, pop_new)
+        return pop_new
 
     def evolve(self, epoch):
         """
@@ -230,10 +240,10 @@ class OppoQSA(BaseQSA):
         Args:
             epoch (int): The current iteration
         """
-        pop = self._update_business_1(self.pop, epoch + 1)
-        pop = self._update_business_2(pop)
-        pop = self._update_business_3(pop, self.g_best)
-        self.pop = self._opposition_based(pop, self.g_best)
+        pop = self.update_business_1__(self.pop, epoch + 1)
+        pop = self.update_business_2__(pop)
+        pop = self.update_business_3__(pop, self.g_best)
+        self.pop = self.opposition_based__(pop, self.g_best)
 
 
 class LevyQSA(BaseQSA):
@@ -257,7 +267,6 @@ class LevyQSA(BaseQSA):
     >>>     "lb": [-10, -15, -4, -2, -8],
     >>>     "ub": [10, 15, 12, 8, 20],
     >>>     "minmax": "min",
-    >>>     "verbose": True,
     >>> }
     >>>
     >>> epoch = 1000
@@ -275,13 +284,13 @@ class LevyQSA(BaseQSA):
             pop_size (int): number of population size, default = 100
         """
         super().__init__(problem, epoch, pop_size, **kwargs)
-        self.nfe_per_epoch = 3 * pop_size
+        self.nfe_per_epoch = 3 * self.pop_size
         self.sort_flag = True
 
-    def _update_business_2(self, pop=None, current_epoch=None):
+    def update_business_2__(self, pop=None, current_epoch=None):
         A1, A2, A3 = pop[0][self.ID_POS], pop[1][self.ID_POS], pop[2][self.ID_POS]
         t1, t2, t3 = pop[0][self.ID_TAR][self.ID_FIT], pop[1][self.ID_TAR][self.ID_FIT], pop[2][self.ID_TAR][self.ID_FIT]
-        q1, q2, q3 = self._calculate_queue_length__(t1, t2, t3)
+        q1, q2, q3 = self.calculate_queue_length__(t1, t2, t3)
         pr = [i / self.pop_size for i in range(1, self.pop_size + 1)]
         if t1 > 1.0e-6:
             cv = t1 / (t2 + t3)
@@ -302,14 +311,18 @@ class LevyQSA(BaseQSA):
                     X_new = pop[i][self.ID_POS] + np.random.normal(0, 1, self.problem.n_dims) * levy_step
                 else:
                     X_new = pop[i][self.ID_POS] + np.random.exponential(0.5) * (A - pop[id1][self.ID_POS])
-                pos_new = self.amend_position_faster(X_new)
+                pos_new = self.amend_position(X_new, self.problem.lb, self.problem.ub)
             else:
-                pos_new = np.random.uniform(self.problem.lb, self.problem.ub)
+                pos_new = self.generate_position(self.problem.lb, self.problem.ub)
+            pos_new = self.amend_position(pos_new, self.problem.lb, self.problem.ub)
             pop_new.append([pos_new, None])
-        pop_new = self.update_fitness_population(pop_new)
-        pop_new = self.greedy_selection_population(pop, pop_new)
-        pop_new, _ = self.get_global_best_solution(pop_new)
-        return pop_new
+            if self.mode not in self.AVAILABLE_MODES:
+                target = self.get_target_wrapper(pos_new)
+                pop_new[-1] = self.get_better_solution([pos_new, target], pop[i])
+        if self.mode in self.AVAILABLE_MODES:
+            pop_new = self.update_target_wrapper_population(pop_new)
+            pop_new = self.greedy_selection_population(pop, pop_new)
+        return self.get_sorted_strim_population(pop_new, self.pop_size)
 
     def evolve(self, epoch):
         """
@@ -318,9 +331,9 @@ class LevyQSA(BaseQSA):
         Args:
             epoch (int): The current iteration
         """
-        pop = self._update_business_1(self.pop, epoch + 1)
-        pop = self._update_business_2(pop, epoch + 1)
-        self.pop = self._update_business_3(pop, self.g_best)
+        pop = self.update_business_1__(self.pop, epoch + 1)
+        pop = self.update_business_2__(pop, epoch + 1)
+        self.pop = self.update_business_3__(pop, self.g_best)
 
 
 class ImprovedQSA(OppoQSA, LevyQSA):
@@ -343,7 +356,6 @@ class ImprovedQSA(OppoQSA, LevyQSA):
     >>>     "lb": [-10, -15, -4, -2, -8],
     >>>     "ub": [10, 15, 12, 8, 20],
     >>>     "minmax": "min",
-    >>>     "verbose": True,
     >>> }
     >>>
     >>> epoch = 1000
@@ -366,7 +378,7 @@ class ImprovedQSA(OppoQSA, LevyQSA):
             pop_size (int): number of population size, default = 100
         """
         super().__init__(problem, epoch, pop_size, **kwargs)
-        self.nfe_per_epoch = 4 * pop_size
+        self.nfe_per_epoch = 4 * self.pop_size
         self.sort_flag = True
 
     def evolve(self, epoch):
@@ -376,10 +388,10 @@ class ImprovedQSA(OppoQSA, LevyQSA):
         Args:
             epoch (int): The current iteration
         """
-        pop = self._update_business_1(self.pop, epoch + 1)
-        pop = self._update_business_2(pop, epoch + 1)
-        pop = self._update_business_3(pop, self.g_best)
-        self.pop = self._opposition_based(pop, self.g_best)
+        pop = self.update_business_1__(self.pop, epoch + 1)
+        pop = self.update_business_2__(pop, epoch + 1)
+        pop = self.update_business_3__(pop, self.g_best)
+        self.pop = self.opposition_based__(pop, self.g_best)
 
 
 class OriginalQSA(BaseQSA):
@@ -402,7 +414,6 @@ class OriginalQSA(BaseQSA):
     >>>     "lb": [-10, -15, -4, -2, -8],
     >>>     "ub": [10, 15, 12, 8, 20],
     >>>     "minmax": "min",
-    >>>     "verbose": True,
     >>> }
     >>>
     >>> epoch = 1000
@@ -425,10 +436,10 @@ class OriginalQSA(BaseQSA):
             pop_size (int): number of population size, default = 100
         """
         super().__init__(problem, epoch, pop_size, **kwargs)
-        self.nfe_per_epoch = 3 * pop_size
+        self.nfe_per_epoch = 3 * self.pop_size
         self.sort_flag = True
 
-    def _update_business_3(self, pop, g_best):
+    def update_business_3__(self, pop, g_best):
         pr = [i / self.pop_size for i in range(1, self.pop_size + 1)]
         pop_new = []
         for i in range(self.pop_size):
@@ -440,10 +451,15 @@ class OriginalQSA(BaseQSA):
                     X1 = pop[i1][self.ID_POS]
                     X2 = pop[i2][self.ID_POS]
                     pos_new[j] = X1[j] + e * (X2[j] - pop[i][self.ID_POS][j])
-            pos_new = self.amend_position_faster(pos_new)
+            pos_new = self.amend_position(pos_new, self.problem.lb, self.problem.ub)
             pop_new.append([pos_new, None])
-        pop_new = self.update_fitness_population(pop_new)
-        return self.greedy_selection_population(pop, pop_new)
+            if self.mode not in self.AVAILABLE_MODES:
+                target = self.get_target_wrapper(pos_new)
+                pop_new[-1] = self.get_better_solution([pos_new, target], pop[i])
+        if self.mode in self.AVAILABLE_MODES:
+            pop_new = self.update_target_wrapper_population(pop_new)
+            pop_new = self.greedy_selection_population(pop, pop_new)
+        return pop_new
 
     def evolve(self, epoch):
         """
@@ -452,6 +468,6 @@ class OriginalQSA(BaseQSA):
         Args:
             epoch (int): The current iteration
         """
-        pop = self._update_business_1(self.pop, epoch)
-        pop = self._update_business_2(pop)
-        self.pop = self._update_business_3(pop, self.g_best)
\ No newline at end of file
+        pop = self.update_business_1__(self.pop, epoch)
+        pop = self.update_business_2__(pop)
+        self.pop = self.update_business_3__(pop, self.g_best)
